@@ -837,14 +837,14 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         return response
 
     def on_delete_event(self):
-        # TODO: This should not be necessary; remove if and when we
-        # figure out what's keeping MeldDocs alive for too long.
-        del self._cached_match
         self.state = melddoc.STATE_CLOSING
         response = self.check_save_modified()
         if response == Gtk.ResponseType.OK:
             for h in self.settings_handlers:
                 meldsettings.disconnect(h)
+            # TODO: This should not be necessary; remove if and when we
+            # figure out what's keeping MeldDocs alive for too long.
+            del self._cached_match
             # TODO: Base the return code on something meaningful for VC tools
             self.emit('close', 0)
         return response
@@ -1142,7 +1142,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             yield i
         for i in self._diff_files():
             yield i
-        focus_pane = 0 if self.num_panes < 3 else 1
+        focus_pane = 0 if self.num_panes < 2 else 1
         self.textview[focus_pane].grab_focus()
 
     def set_meta(self, meta):
@@ -1481,38 +1481,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             msgarea.show_all()
             return False
 
-        start, end = buf.get_bounds()
-        text = buf.get_text(start, end, False)
-
-        source_encoding = bufdata.sourcefile.get_encoding()
-        if not source_encoding:
-            # no encoding for new blank comparison
-            source_encoding = GtkSource.Encoding.get_utf8()
-
-        while isinstance(text, str):
-            try:
-                encoding = source_encoding.get_charset()
-                text = text.encode(encoding)
-            except UnicodeEncodeError:
-                dialog_buttons = [
-                    (_("_Cancel"), Gtk.ResponseType.CANCEL),
-                    (_("_Save as UTF-8"), Gtk.ResponseType.OK),
-                ]
-                reencode = misc.modal_dialog(
-                    primary=_(u"Couldn't encode text as “%s”") % encoding,
-                    secondary=_(
-                        u"File “%s” contains characters that can't be encoded "
-                        u"using encoding “%s”.\n"
-                        u"Would you like to save as UTF-8?") % (
-                        bufdata.label, encoding),
-                    buttons=dialog_buttons,
-                    messagetype=Gtk.MessageType.WARNING
-                )
-                if reencode != Gtk.ResponseType.OK:
-                    return False
-
-                source_encoding = GtkSource.Encoding.get_utf8()
-
         saver = GtkSource.FileSaver.new_with_target(
             self.textbuffer[pane], bufdata.sourcefile, bufdata.gfiletarget)
         # TODO: Think about removing this flag and above handling, and instead
